
;
;   handleuc.inc
;
;   This file contains routines which handle a C START during main loop
;   and handle the received <C ...> packet according to type
;
;   last modified: 06/07/2002 mmddyy 02:41 by Burkart
;
; List of routines:
;
; - call handle_main_start
; - call handle_uc_re_started
; - call handle_ping
; - call handle_uc_logged_out
; - call handle_uc_sent_radio_packet
; - call handle_uc_received_radio_packet
; - call handle_uc_received_complete_radio_packet
;


;-----------------------------------------------
; call handle_main_start
;-----------------------------------------------
; call this if a START was received in the main
; loop and now has to be handled
;-----------------------------------------------

handle_main_start:
        call    wait_for_start_with_timeout     ; receive START
        jp      c, error_start_timedout         ; DEVELOPMENT
        call    receive_packet                  ; receive C packet
        
        ld      hl, tiucpacket_ping             ; check for <C ping>
        ld      b, 3                            ; the first 3 byte have to be identical
        call    check_received_packet           ; perform the comparison
        jp      nc, handle_ping                 ; identical -> handle packet

        ld      hl, tiucpacket_uc_logged_out    ; check for <C logged out>
        ld      b, 3                            ; the first 3 byte have to be identical
        call    check_received_packet           ; perform the comparison
        jp      nc, handle_uc_logged_out        ; identical -> handle packet

        ld      hl, tiucpacket_uc_sent_radio_packet ; check for <C sent radio packet>
        ld      b, 3                            ; the first 3 byte have to be identical
        call    check_received_packet           ; perform the comparison
        jp      nc, handle_uc_sent_radio_packet ; identical -> handle packet

        ld      hl, tiucpacket_uc_received_radio_packet ; check for <C received radio packet>
        ld      b, 3                            ; the first 3 byte have to be identical
        call    check_received_packet           ; perform the comparison
        jp      nc, handle_uc_received_radio_packet ; identical -> handle packet

        ld      hl, tiucpacket_uc_received_complete_radio_packet ; check for <C received complete radio (multi-)packet>
        ld      b, 3                            ; the first 3 byte have to be identical
        call    check_received_packet           ; perform the comparison
        jp      nc, handle_uc_received_complete_radio_packet ; identical -> handle packet

        ld      hl, tiucpacket_uc_re_started    ; check for <C (re)started>
        ld      b, 3                            ; the first 3 byte have to be identical
        call    check_received_packet           ; perform the comparison
        jp      nc, handle_uc_re_started        ; identical -> handle packet

        ; any strange packet

        ret                                     ; DEVELOPMENT: return to caller

;-----------------------------------------------
; call handle_uc_re_started
;-----------------------------------------------
; <C (re)started> has been received, this routine
; handles it
; when done, EXIT_2_TIOS
;-----------------------------------------------

handle_uc_re_started:
        call    _clrLCD                         ; clear LCD
        ld      hl, $0000                       ; reset cursor
        ld      (_curPos), hl
        ld      hl, str_fatal_error             ; display error message
        call    _puts
        ld      hl, str_uc_restarted
        call    _puts
        ld      hl, str_press_any_key
        call    _puts
        call    wait_for_keypress               ; wait for keypress
        jp      EXIT_2_TIOS                     ; exit to TIOS

;-----------------------------------------------
; call handle_ping
;-----------------------------------------------
; <C ping> has been received, this routine
; handles it by sending <TI ACK>
;-----------------------------------------------

handle_ping:
        ; send <TI ACK>:

        call    send_start                      ; send START
        ld      hl, tiucpacket_ack              ; template packet
        ld      b, 3                            ; consists of 3 bytes
        call    send_packet_to_uc               ; send to C

        ret                                     ; RETurn to caller

;-----------------------------------------------
; call handle_uc_logged_out
;-----------------------------------------------
; <C logged out> has been received, this routine
; handles it by sending <TI ACK> and displaying
; "Disconnected"
; when done, after the user has pressed a key,
; control is returned to the shell
;-----------------------------------------------

handle_uc_logged_out:
        ; send <TI ACK>:

        call    send_start                      ; send START
        ld      hl, tiucpacket_ack              ; template packet
        ld      b, 3                            ; consists of 3 bytes
        call    send_packet_to_uc               ; send to C

        call    _clrLCD                         ; clear LCD
        ld      hl, $0503                       ; move cursor
        ld      (_curPos), hl
        ld      hl, str_disconnected            ; display "Disconnected"
        call    _puts

        pop     af                              ; clear up stack (caller address)
        jp      wait_for_keypress               ; wait for keypress, then RET

;-----------------------------------------------
; call handle_uc_sent_radio_packet
;-----------------------------------------------
; <C sent radio packet> has been received, this
; routine handles it
;-----------------------------------------------

handle_uc_sent_radio_packet:
        ; send <TI ACK>:

        call    send_start                      ; send START
        ld      hl, tiucpacket_ack              ; template packet
        ld      b, 3                            ; consists of 3 bytes
        call    send_packet_to_uc               ; send to C

        call    _clrLCD                         ; clear LCD
        ld      hl, $0903                       ; move cursor
        ld      (_curPos), hl
        ld      hl, str_sent                    ; display "Sent"
        call    _puts

        ret                                     ; RETurn to caller

;-----------------------------------------------
; call handle_uc_received_radio_packet
;-----------------------------------------------
; <C received radio packet> has been received,
; this routine handles it
;-----------------------------------------------

handle_uc_received_radio_packet:
        ; send <TI ACK>:

        call    send_start                      ; send START
        ld      hl, tiucpacket_ack              ; template packet
        ld      b, 3                            ; consists of 3 bytes
        call    send_packet_to_uc               ; send to C

        xor     a                               ; flag_new_radio_packet := true
        ld      (var_flag_new_radio_packet), a
        cpl                                     ; flag_clear_to_receive := false
        ld      (var_flag_clear_to_receive), a

        ret                                     ; return to caller

;-----------------------------------------------
; call handle_uc_received_complete_radio_packet
;-----------------------------------------------
; <C received complete radio (multi-)packet>
; has been received, this routine handles it
;-----------------------------------------------

handle_uc_received_complete_radio_packet:
        ; send <TI ACK>:

        call    send_start                      ; send START
        ld      hl, tiucpacket_ack              ; template packet
        ld      b, 3                            ; consists of 3 bytes
        call    send_packet_to_uc               ; send to C

        ; data at buf_received_packet:
        ; 1 word length
        ; 1 byte 52h
        ; 1 byte Content-ID
        ; 1 word data length

        ld      hl, buf_received_packet+3       ; get Content-ID and data length
        ld      de, var_radio_in_content_id
        ld      bc, 3
        ldir

        xor     a                               ; flag_clear_to_receive := true
        ld      (var_flag_clear_to_receive), a
        ld      (var_flag_new_radio_packet), a  ; flag_new_radio_packet := true

        ret                                     ; return to caller

