
;
;   uc-comm.inc
;
;   This file contains routines to communicate with
;   the microcontroller using TI<->uC LLP and HLP
;
;   last modified: 06/07/2002 mmddyy 01:26 by Burkart
;
;
; List of routines:
;
; - call check_received_packet
; - call receive_packet
; - call receive_byte
; - jp   error_receive_buffer_too_small
; - jp   error_receive_bit_timeout
; - jp   transmit_bin0_timeout
; - jp   transmit_bin1_timeout
; - jp   transmit_bit_ready_timeout
; - jp   error_start_timeout
; - jp   error_start_timedout
; - jp   error_packets_differ
; - call send_start
; - call wait_for_start_with_timeout
; - call wait_for_start_no_timeout
; - call send_packet_to_uc
; - call send_byte_to_uc
;


;-----------------------------------------------
; call check_received_packet
;-----------------------------------------------
; compares the received packet and a template
; packet (the first b bytes of them)
;-----------------------------------------------
; input: hl - pointer to template packet
;        b  - number of bytes to be checked
;        received packet at (buf_received_packet)
;-----------------------------------------------
; output: CY=0 - packets are identical
;         CY=1 - packets are different
;-----------------------------------------------
; modified: identical: b=0, hl=hl+b
;           different: b, hl
;-----------------------------------------------
; stack: 4 byte + caller address
;-----------------------------------------------

check_received_packet:
        push    af                              ; save used registers
        push    de

        ld      de, buf_received_packet         ; let de point to received packet
check_received_packet_loop:
        ld      a, (de)                         ; compare byte of template and received packet
        cp      (hl)                            ; identical?
        jr      nz, check_received_packet_different     ; no, set CY and exit
        inc     hl                              ; advance to next byte
        inc     de
        djnz    check_received_packet_loop      ; yes, next byte
        
        ; packets are identical

        pop     de                              ; clear up stack (saved registers)
        pop     af

        scf                                     ; CY = 0
        ccf
        ret                                     ; return to caller

check_received_packet_different:
        pop     de                              ; clear up stack (saved registers)
        pop     af

        scf                                     ; CY = 1
        ret                                     ; return to caller

;-----------------------------------------------
; call receive_packet
;-----------------------------------------------
; Receives packet via TI<->C Protocol and stores
; it at buf_received_packet
; START must have been received in advance
;-----------------------------------------------
; modified: N/A (af in case of error)
;-----------------------------------------------
; stack: 10 byte + caller address
;        + 10 byte (receive_byte)
;-----------------------------------------------

receive_packet:
        push    af                              ; save used registers
        push    hl
        push    bc
        push    de

        call    receive_byte                    ; receive length of packet, store it and save it in hl
        ld      (buf_received_packet), a
        ld      l, a

        call    receive_byte
        ld      (buf_received_packet+1), a
        ld      h, a

        jr      receive_packet_buffer_big_enough ; DEBUG CODE

;        ld      de, const_received_packet_maxlength     ; is buffer big enough?
;;       call    _cphlde ; cp hl, de
;        ld      a, h
;        cp      d
;        jr      c, receive_packet_buffer_too_small      ; no, cancel transfer
;        jr      nz, receive_packet_buffer_big_enough    ; yes, go on
;        ; maybe (hi-lengthes identical)
;        ld      a, l
;        cp      e
;        jr      nc, receive_packet_buffer_big_enough    ; yes, go on
;
;receive_packet_buffer_too_small:
;        ; buffer is too small
;
;        pop     de                              ; clear up stack (saved registers)
;        pop     bc
;        pop     hl
;        pop     af
;        pop     af                              ; clear up stack (caller address)
;        jr      error_receive_buffer_too_small  ; jump to error_handler

receive_packet_buffer_big_enough:
        ld      d, h                            ; save buffer length to de
        ld      e, l
        ld      hl, buf_received_packet+2       ; let hl point to next byte of buffer

;        ld      a, d                            ; check if hi-length <> 0
;        or      a
;        jr      nz, receive_packet_length_hi_not_zero
;
;        ; hi-length is zero
;
;receive_packet_check_lo_length_for_zero:
;        ld      a, e                            ; check if lo-length <> 0
;        or      a
;        jr      nz, receive_packet_length_hi_zero_lo_not_zero
;
;        ; packet length is zero or $xx00 with $xx byte already received
;
;        pop     de                              ; clear up stack (saved registers)
;        pop     bc
;        pop     hl
;        pop     af
;        ret                                     ; return to caller
;
;receive_packet_length_hi_not_zero:
;        ld      b, d                            ; receive 256 x hi-len byte
;receive_packet_loop_hi:
;        push    bc
;        ld      b, $00
;receive_packet_loop_256:
;        call    receive_byte                    ; receive byte
;        ld      (hl), a                         ; save it to buffer
;        inc     hl                              ; advance buffer pointer
;        djnz    receive_packet_loop_256
;        pop     bc
;        djnz    receive_packet_loop_hi
;
;        jr      receive_packet_check_lo_length_for_zero ; let's see if there are still some (<= 255) byte to receive
;
;receive_packet_length_hi_zero_lo_not_zero:
;        call    receive_byte                    ; receive byte
;        ld      (hl), a                         ; save it to buffer
;        inc     hl                              ; advance buffer pointer
;        djnz    receive_packet_length_hi_zero_lo_not_zero

;----------
        dec     hl

        xor     a                               ; MSB of length=0 -> receive (LSB) byte
        cp      d
        jr      z, R_GetLowerLengthByte

        ld      b, d                            ; receive (MSB) ...
R_GetHigherLengthByte:
        push    bc                              ; ...times 256 byte
        ld      b, 0
R_Get256Byte:
        call    receive_byte                    ; receive byte
        inc     hl                              ; move buffer pointer
        ld      (hl), a                         ; store received byte into buffer
        djnz    R_Get256Byte                    ; receive next byte
        pop     bc
        djnz    R_GetHigherLengthByte

R_GetLowerLengthByte:
        xor     a
        cp      e                               ; LSB of length=0 -> finished receiving
        jr      z, R_PacketReceived

        ld      b, e                            ; receive (LSB) byte
R_GetLowerLengthByte2:
        call    receive_byte                    ; receive byte
        inc     hl                              ; move buffer pointer
        ld      (hl), a                         ; store it into buffer
        djnz    R_GetLowerLengthByte2           ; receive next byte

R_PacketReceived:
;----------

        pop     de                              ; clear up stack (saved registers)
        pop     bc
        pop     hl
        pop     af
        ret                                     ; return to caller

;-----------------------------------------------
; call receive_byte
;-----------------------------------------------
; Receives Byte via TI<->C Protocol and stores
; it into a
;-----------------------------------------------
; output: a - received byte
;-----------------------------------------------
; modified: (a), f
;-----------------------------------------------
; stack: 8 byte + caller address
;-----------------------------------------------

receive_byte:
        push    de                              ; save used registers
        push    bc

        ld      b, 8                            ; receive 8 bits
receive_byte_receive_bit:
        push    bc
        ld      b, $80
receive_byte_timeoutloop1:
        push    bc
        ld      b, $00
receive_byte_timeoutloop2:
        in      a, (P_LINK)                     ; get linkport data
        sra     a                               ; red wire = 0V -> binary 1 received
        jr      nc, receive_byte_received_bin1
        sra     a                               ; white wire = 0V -> binary 0 received
        jr      nc, receive_byte_received_bin0

        djnz    receive_byte_timeoutloop2
        pop     bc
        djnz    receive_byte_timeoutloop1
        pop     bc

        ; timeout

        pop     bc                              ; clear up stack (saved registers)
        pop     de
        pop     bc                              ; clear up stack (caller address)
        jr      error_receive_bit_timeout       ; jump to error_handler

receive_byte_received_bin1:
        pop     bc                              ; clear up stack (loop counters)
        pop     bc

        scf
        ; CY = 1
        rl      d                               ; store binary 1 into d

#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r1w0                         ; white wire = 0V (ACK)
        out     (P_LINK), a

receive_byte_wait_red5_2:
        in      a, (P_LINK)                     ; wait until red wire = 5V (no timeout)
        sra     a
        jr      nc, receive_byte_wait_red5_2

        jr      receive_byte_bit_received_completely

receive_byte_received_bin0:
        pop     bc                              ; clear up stack (loop counters)
        pop     bc

        ; CY = 0
        rl      d                               ; store binary 0 into d

#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r0w1                         ; red wire = 0V (ACK)
        out     (P_LINK), a

receive_byte_wait_white5:
        in      a, (P_LINK)                     ; wait until white wire = 5V (no timeout)
        sra     a
        sra     a
        jr      nc, receive_byte_wait_white5

        ;jr     receive_byte_bit_received_completely

receive_byte_bit_received_completely:
#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r1w1                         ; red wire = 5V; white wire = 5V
        out     (P_LINK), a

        djnz    receive_byte_receive_bit        ; receive all 8 bits

        ld      a, d                            ; store byte into a

        pop     bc                              ; clear up stack (used registers)
        pop     de
        ret                                     ; return to caller

;-----------------------------------------------
; jp error_receive_buffer_too_small
;-----------------------------------------------
; Jump here if a received packet is bigger than
; the receive buffer
;-----------------------------------------------

error_receive_buffer_too_small:
        call    _clrLCD                         ; clear LCD
        ld      hl, $0000                       ; reset cursor
        ld      (_curPos), hl
        ld      hl, str_fatal_error             ; display "* Fatal error! *"
        call    _puts
        ld      hl, str_receive_buffer_too_small; display "receive buffer  |too small       "
        call    _puts
        ld      hl, str_press_any_key           ; display "Press any key"
        call    _puts
        call    wait_for_keypress               ; wait for keypress

        jp      EXIT_2_TIOS                     ; DEVELOPMENT: exit program and CrASH

;-----------------------------------------------
; jp error_receive_bit_timeout
;-----------------------------------------------
; Jump here if an error occured during receiving
; of a bit
;-----------------------------------------------

error_receive_bit_timeout:
        call    _clrLCD                         ; clear LCD
        ld      hl, $0000                       ; reset cursor
        ld      (_curPos), hl
        ld      hl, str_fatal_error             ; display "* Fatal error! *"
        call    _puts
        ld      hl, str_uc_communication        ; "uC communication"
        call    _puts
        ld      hl, str_receive_bit_timeout     ; display "recv. bit t.out "
        call    _puts
        ld      hl, str_press_any_key           ; display "Press any key"
        call    _puts
        call    wait_for_keypress               ; wait for keypress

        jp      EXIT_2_TIOS                     ; DEVELOPMENT: exit program and CrASH

;-----------------------------------------------
; jp transmit_..._timeout
;-----------------------------------------------
; Jump here if an error occured during sending
; of a byte
;-----------------------------------------------

transmit_bin0_timeout:
transmit_bin1_timeout:
        call    _clrLCD                         ; clear LCD
        ld      hl, $0000                       ; reset cursor
        ld      (_curPos), hl
        ld      hl, str_fatal_error             ; display "* Fatal error! *"
        call    _puts
        ld      hl, str_uc_communication        ; "uC communication"
        call    _puts
        ld      hl, str_transmit_binx_timeout   ; display "transmit bin 0/1|timeout         "
        call    _puts
        ld      hl, str_press_any_key           ; display "Press any key"
        call    _puts
        call    wait_for_keypress               ; wait for keypress

        jp      EXIT_2_TIOS                     ; DEVELOPMENT: exit program and CrASH

transmit_bit_ready_timeout:
        call    _clrLCD                         ; clear LCD
        ld      hl, $0000                       ; reset cursor
        ld      (_curPos), hl
        ld      hl, str_fatal_error             ; display "* Fatal error! *"
        call    _puts
        ld      hl, str_uc_communication        ; "uC communication"
        call    _puts
        ld      hl, str_transmit_bit_ready_timeout      ; display "transmit bit    |ready timeout   "
        call    _puts
        ld      hl, str_press_any_key           ; display "Press any key"
        call    _puts
        call    wait_for_keypress               ; wait for keypress

        jp      EXIT_2_TIOS                     ; DEVELOPMENT: exit program and CrASH
        
;-----------------------------------------------
; jp error_start_timeout
;-----------------------------------------------
; Jump here if an error occured during sending
; of START
;-----------------------------------------------

error_start_timeout:
        call    _clrLCD                         ; clear LCD
        ld      hl, $0000                       ; reset cursor
        ld      (_curPos), hl
        ld      hl, str_fatal_error             ; display "* Fatal error! *"
        call    _puts
        ld      hl, str_uc_communication        ; "uC communication"
        call    _puts
        ld      hl, str_send_start_timeout      ; display "send START t.out"
        call    _puts
        ld      hl, str_press_any_key           ; display "Press any key"
        call    _puts
        call    wait_for_keypress               ; wait for keypress

        jp      EXIT_2_TIOS                     ; DEVELOPMENT: exit program and CrASH

;-----------------------------------------------
; jp error_start_timedout
;-----------------------------------------------
; Jump here if a timeout occured during reception
; of START in wait_for_start_with_timeout and
; you don't know how to handle it
;-----------------------------------------------

error_start_timedout:
        call    _clrLCD                         ; clear LCD
        ld      hl, $0000                       ; reset cursor
        ld      (_curPos), hl
        ld      hl, str_fatal_error             ; display "* Fatal error! *"
        call    _puts
        ld      hl, str_uc_communication        ; "uC communication"
        call    _puts
        ld      hl, str_recv_start_timeout      ; display "recv START t.out"
        call    _puts
        ld      hl, str_press_any_key           ; display "Press any key"
        call    _puts
        call    wait_for_keypress               ; wait for keypress

        jp      EXIT_2_TIOS                     ; DEVELOPMENT: exit program and CrASH

;-----------------------------------------------
; jp error_packets_differ
;-----------------------------------------------
; Jump here if check_received_packet returned
; CY=1 (packets different) and you don't know
; how to handle it
;-----------------------------------------------

error_packets_differ:
        call    _clrLCD                         ; clear LCD
        ld      hl, $0000                       ; reset cursor
        ld      (_curPos), hl
        ld      hl, str_fatal_error             ; display "* Fatal error! *"
        call    _puts
        ld      hl, str_uc_communication        ; "uC communication"
        call    _puts
        ld      hl, str_packet_different        ; display "packet different"
        call    _puts
        ld      hl, str_press_any_key           ; display "Press any key"
        call    _puts
        call    wait_for_keypress               ; wait for keypress

        jp      EXIT_2_TIOS                     ; DEVELOPMENT: exit program and CrASH

;-----------------------------------------------
; call send_start
;-----------------------------------------------
; send START according to TI<->C Protocol
; if a timeout occurs, a jump to error_start_timeout
; is performed
;-----------------------------------------------
; modified: N/A; af in case of error
;-----------------------------------------------
; stack: 6 byte + caller address
;-----------------------------------------------

send_start:
        push    af                              ; save used registers
        push    bc

#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r0w1                         ; red wire = 0V (START)
        out     (P_LINK), a

        ld      b, $80
send_start_timeoutloop1a
        push    bc
        ld      b, $00
send_start_timeoutloop1b:
        in      a, (P_LINK)                     ; get linkport data
        sra     a                               ; white wire = 0V -> START was Acknowledged
        sra     a
        jr      nc, send_start_start_acked1

        djnz    send_start_timeoutloop1b
        pop     bc
        djnz    send_start_timeoutloop1a

send_start_timeout:
        ; Timeout

#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r1w1                         ; both lines = 5V
        out     (P_LINK), a

        pop     bc                              ; clear up stack (saved registers)
        pop     af
        pop     af                              ; clear up stack (caller address)
        jp      error_start_timeout             ; jump to error handler

send_start_start_acked1:
        pop     bc
        ; START was ACKed, now both wires are 0V

#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r1w1                         ; red wire = 5V (end of START)
        out     (P_LINK), a

        ld      b, $80
send_start_timeoutloop2a:
        push    bc
        ld      b, $00
send_start_timeoutloop2b:
        in      a, (P_LINK)                     ; wait until white wire = 5V
        sra     a
        sra     a
        jr      c, send_start_start_acked2

        djnz    send_start_timeoutloop2b
        pop     bc
        djnz    send_start_timeoutloop2a
        
        jr      send_start_timeout

send_start_start_acked2:
        pop     bc
        ; START was successfully sent
        ; both wires are 5V

        pop     bc                              ; clear up stack (saved registers)
        pop     af
        ret                                     ; return to caller

;-----------------------------------------------
; call wait_for_start_with_timeout
;-----------------------------------------------
; wait until START is received
; if a timeout occurs, CY is set, otherwise not
;-----------------------------------------------
; output: CY=1: timeout, START not received
;         CY=0: no timeout, START received
;-----------------------------------------------
; modified: N/A; af in case of error
;-----------------------------------------------
; stack: 6 byte + caller address
;-----------------------------------------------

wait_for_start_with_timeout:
        push    af                              ; save used registers
        push    bc

        ld      b, $80
wait_for_start_with_timeout_timeoutloop1a:
        push    bc
        ld      b, $00
wait_for_start_with_timeout_timeoutloop1b:        
        in      a, (P_LINK)                     ; check link port
        sra     a                               ; red wire = 0V -> START
        jr      nc, wait_for_start_with_timeout_ok1

        djnz    wait_for_start_with_timeout_timeoutloop1b
        pop     bc
        djnz    wait_for_start_with_timeout_timeoutloop1a

        ; timeout

wait_for_start_with_timeout_timeout:
        pop     bc                              ; clear up stack (saved registers)
        pop     af

        scf                                     ; set CY in order to express error
        ret                                     ; return to caller

wait_for_start_with_timeout_ok1:
        pop     bc                              ; clear up stack (loop counter)

        ; START was received and now has to be handled

#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r1w0                         ; white wire = 0V (ACK)
        out     (P_LINK), a

        ld      b, $80
wait_for_start_with_timeout_timeoutloop2a:
        push    bc
        ld      b, $00
wait_for_start_with_timeout_timeoutloop2b:        
        in      a, (P_LINK)                     ; check link port
        sra     a                               ; red wire = 5V -> ACK was ACKed
        jr      c, wait_for_start_with_timeout_ok2

        djnz    wait_for_start_with_timeout_timeoutloop2b
        pop     bc
        djnz    wait_for_start_with_timeout_timeoutloop2a

        ; timeout

        jr      wait_for_start_with_timeout_timeout     ; handle timeout

wait_for_start_with_timeout_ok2:
        pop     bc                              ; clear up stack (loop counter)

#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r1w1                         ; both wires = 5V
        out     (P_LINK), a

        ; START was received successfully

        pop     bc                              ; clear up stack (saved registers)
        pop     af

        scf                                     ; clear CY
        ccf
        ret                                     ; return to caller


;-----------------------------------------------
; call wait_for_start_no_timeout
;-----------------------------------------------
; wait until START is received
; no timeout!!!
;-----------------------------------------------
; modified: N/A
;-----------------------------------------------
; stack: 2 byte + caller address
;-----------------------------------------------

wait_for_start_no_timeout:
        push    af                              ; save used registers

wait_for_start_no_timeout_wait1:
        in      a, (P_LINK)                     ; check link port
        sra     a                               ; red wire = 0V -> START
        jr      c, wait_for_start_no_timeout_wait1

        ; START was received and now has to be handled

#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r1w0                         ; white wire = 0V (ACK)
        out     (P_LINK), a

wait_for_start_no_timeout_wait2:
        in      a, (P_LINK)                     ; check link port
        sra     a                               ; red wire = 5V -> ACK was ACKed
        jr      nc, wait_for_start_no_timeout_wait2

        ; START was received successfully

#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r1w1                         ; both wires = 5V
        out     (P_LINK), a

        pop     af                              ; clear up stack (used registers)
        ret                                     ; return to caller

;-----------------------------------------------
; call send_packet_to_uc
;-----------------------------------------------
; transmits a packet via TI<->C Protocol
;-----------------------------------------------
; input: hl - pointer to template packet
;        b  - number of byte to be sent
;-----------------------------------------------
; modified: hl=hl+b, b=0
;-----------------------------------------------
; stack: 2 byte + caller address
;        + 10 byte (send_byte_to_uc)
;-----------------------------------------------

send_packet_to_uc:
        push    af                              ; save used variables

send_packet_to_uc_loop:
        ld      a, (hl)                         ; send packet byte-wise
        call    send_byte_to_uc
        inc     hl                              ; advance to next byte
        djnz    send_packet_to_uc_loop          ; send complete packet

        pop     af                              ; clean up stack (used variables)
        ret                                     ; return to caller

;-----------------------------------------------
; call send_byte_to_uc
;-----------------------------------------------
; transmits a byte via TI<->C Protocol
; if a timeout occurs, a jump to one of the
; following error_handlers is done:
; - transmit_bin0_timeout
; - transmit_bin1_timeout
; - transmit_bit_ready_timeout
;-----------------------------------------------
; input: A - byte to send
;-----------------------------------------------
; modified: N/A (af in case of timeout)
;-----------------------------------------------
; stack: 8 byte + caller address
;-----------------------------------------------

send_byte_to_uc:
        push    af                              ; save used registers
        push    bc

        ld      c, a                            ; move byte-to-send to c

        ld      b, 8                            ; 8 bits have to be sent
send_byte_to_uc_transmit_bit:
        rl      c                               ; is next bit binary 0 or 1?
        jr      c, send_byte_to_uc_transmit_binary1

send_byte_to_uc_transmit_binary0:
#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r1w0                         ; white wire = 0V (binary 0)
        out     (P_LINK), a

        push    bc ; timeout loop a1
        ld      b, $80
send_byte_to_uc_bin0_wait_ack1:
        push    bc ; timeout loop a2
        ld      b, $00
send_byte_to_uc_bin0_wait_ack2:
        in      a, (P_LINK)                     ; wait until red wire = 0V
        sra     a
        jr      nc, send_byte_to_uc_bin0_wait_acked
        djnz    send_byte_to_uc_bin0_wait_ack2
        pop     bc ; timeout loop a2
        djnz    send_byte_to_uc_bin0_wait_ack1

        ; timeout

#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r1w1                         ; both lines = 5V
        out     (P_LINK), a

        pop     bc ; timeout loop a1

        pop     bc                              ; clean up stack (saved registers)
        pop     af
        pop     af                              ; clean up stack (caller address)
        jp      transmit_bin0_timeout           ; jump to error_handler

send_byte_to_uc_bin0_wait_acked:
        ; both lines are now 0V
        pop     bc ; timeout loop a2
        pop     bc ; timeout loop a1
        
send_byte_to_uc_bit_ready
#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r1w1                         ; both lines = 5V
        out     (P_LINK), a

        push    bc ; timeout loop b1
        ld      b, $80
send_byte_to_uc_bit_ready_wait1:
        push    bc ; timeout loop b2
        ld      b, $00
send_byte_to_uc_bit_ready_wait2:
        in      a, (P_LINK)
        sra     a                               ; red wire = 0V -> not ACKed
        jr      nc, send_byte_to_uc_bit_ready_false
        ; white wire = 5V
        sra     a                               ; white wire = 5V -> ACKed
        jr      c, send_byte_to_uc_bit_ready_ok
send_byte_to_uc_bit_ready_false:
        ; white wire = 0V
        djnz    send_byte_to_uc_bit_ready_wait2
        pop     bc ; timeout loop b2
        djnz    send_byte_to_uc_bit_ready_wait1

        ; timeout

        pop     bc ; timeout loop b1

        pop     bc                              ; clean up stack (saved registers)
        pop     af
        pop     bc                              ; clean up stack (caller address)
        jp      transmit_bit_ready_timeout      ; jump to error_handler
        
send_byte_to_uc_bit_ready_ok:
        ; both lines are 5V again, bit transmission successful
        pop     bc ; timeout loop b2
        pop     bc ; timeout loop b1

        ; now the number of remaining bits to transmit is in b again
        djnz    send_byte_to_uc_transmit_bit    ; transmit next bit

        ; all bits transmitted
        pop     bc                              ; clean up stack (saved registers)
        pop     af
        ret                                     ; return to caller


send_byte_to_uc_transmit_binary1:
#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r0w1                         ; red wire = 0V (binary 1)
        out     (P_LINK), a

        push    bc ; timeout loop c1
        ld      b, $80
send_byte_to_uc_bin1_wait_ack1:
        push    bc ; timeout loop c2
        ld      b, $00
send_byte_to_uc_bin1_wait_ack2:
        in      a, (P_LINK)                     ; wait until white wire = 0V
        sra     a
        sra     a
        jr      nc, send_byte_to_uc_bin1_wait_acked
        djnz    send_byte_to_uc_bin1_wait_ack2
        pop     bc ; timeout loop c2
        djnz    send_byte_to_uc_bin1_wait_ack1

        ; timeout

#ifdef _USE_DELAYS
        call    debug_wait
#endif

        ld      a, r1w1                         ; both lines = 5V
        out     (P_LINK), a

        pop     bc ; timeout loop c1

        pop     bc                              ; clean up stack (saved registers)
        pop     af
        pop     af                              ; clean up stack (caller address)
        jp      transmit_bin1_timeout           ; jump to error_handler


send_byte_to_uc_bin1_wait_acked
        ; both lines are now 0V
        pop     bc ; timeout loop c2
        pop     bc ; timeout loop c1

        jr      send_byte_to_uc_bit_ready

